"""Main entrypoint for the Flask application."""

from flask import Flask, render_template, redirect, request, url_for
from flask_caching import Cache
from scraper import FTPScraper, FTLiveScraper
import models

# pylint: disable=invalid-name ; These module-level variables are standard for Flask.
app = Flask(__name__)
cache = Cache(app, config={'CACHE_TYPE': 'simple'})

DISPLAY_DATETIME_FORMAT = '%A, %B %d, %Y at %-I:%M %p'
DISPLAY_DATE_FORMAT = '%A, %B %-d, %Y'
DISPLAY_TIME_FORMAT = '%-I:%M %p on %A'


def _make_cache_key():
    """Create a cache key for Flask-Caching."""
    path = request.path
    args = str(hash(frozenset(request.args.items())))
    return (path + args).encode('utf-8')


@app.after_request
def add_header(response):
    """Add an HTTP response header for cache invalidation."""
    response.cache_control.max_age = 300
    return response


@app.template_filter('strftime')
def _jinja2_filter_datetime(datetime, date=True, time=True):
    """Format a DateTime for display to a user."""
    return datetime.strftime(DISPLAY_DATETIME_FORMAT if date and time
                             else DISPLAY_DATE_FORMAT if date
                             else DISPLAY_TIME_FORMAT if time else '')


@app.route("/")
@cache.cached(timeout=300)
def index():
    """Render the app landing page."""
    ftl_scraper = FTLiveScraper()
    return render_template('index.html',
                           tournaments=ftl_scraper.list_tournaments())


@app.route("/live")
@cache.cached(timeout=300, key_prefix=_make_cache_key)
def live():
    """Render the primary view of live tournament stats."""
    results_url = request.args.get('results_url')
    ftl_id = request.args.get('ftl_id')

    if ftl_id:
        tournament = FTLiveScraper().scrape_tournament(tournament_id=ftl_id)
    elif results_url:
        tournament = FTPScraper(results_url).scrape_tournament()
    else:
        return redirect(url_for('index'))

    return render_template('live.html', tournament=tournament, phases=models.EventPhase)


if __name__ == "__main__":
    app.run()
