"""Domain-specific class definitions."""

from collections import namedtuple
from dataclasses import dataclass, field
from datetime import datetime
from typing import List
from enum import Enum

Fencer = namedtuple('Fencer', 'name is_checked_in ftl_id', defaults=[None])
setattr(Fencer, '__eq__', lambda f1, f2: f1.name == f2.name)


@dataclass
class Tournament:
    """A fencing competition with live results powered by Fencing Time."""

    name: str
    url: str
    ftl_id: str = None  # type: ignore
    updated: datetime = None  # type: ignore
    location: str = None  # type: ignore
    events: List['Event'] = field(default_factory=list)

    def count_fencers(self):
        """Count the fencers in all events."""
        for event in self.events:
            event.count_fencers()


class EventPhase(Enum):
    """The current state of an Event."""

    def __str__(self):
        """Provide the readable name when used as a string."""
        return str(self.name)
    REGISTRATION = "Registration"
    STARTED = "Started"
    FINISHED = "Finished"


@dataclass
class EventStatistics:
    """Fencer counts for an event."""

    fencers_checked_in: List[Fencer] = field(default_factory=list)
    new_fencers_not_checked_in: List[Fencer] = field(default_factory=list)
    previously_fenced: dict = field(default_factory=dict)
    previous_total: int = 0


@dataclass
class Event:  # pylint: disable=too-many-instance-attributes
    """A single event in a tournament (e.g. Y12 Mens Foil)."""

    name: str
    url: str
    time: datetime
    tournament: Tournament
    phase: EventPhase
    ftl_id: str = None  # type: ignore
    stats: EventStatistics = field(default_factory=EventStatistics)
    fencers: List[Fencer] = field(default_factory=list)

    def __repr__(self):
        """Use event name as the canonical representation."""
        return self.name

    def count_fencers(self):
        """Count the fencers in an event.

        Result is divided by status (checked in, not checked in, etc.)
        """
        self.stats.fencers_checked_in = []
        self.stats.new_fencers_not_checked_in = []
        self.stats.previously_fenced = {}
        self.stats.previous_total = 0
        for fencer in self.fencers:
            if fencer.is_checked_in:
                self.stats.fencers_checked_in.append(fencer)
            else:
                self.stats.new_fencers_not_checked_in.append(fencer)
            for event in self.tournament.events:
                if event.name == self.name:
                    break
                if fencer in event.fencers:
                    if event.name in self.stats.previously_fenced:
                        self.stats.previously_fenced[event.name] += 1
                    else:
                        self.stats.previously_fenced[event.name] = 1
                    self.stats.previous_total += 1
                    try:
                        self.stats.new_fencers_not_checked_in.remove(fencer)
                    except ValueError:
                        pass  # already removed; ignore
                    break
